// --------------------------------------------------------------------------------------
//
// RobStanzeMain.CPP                                               Ulrich Mller 11.06.02
//                                                                 PCS1700   C++Builder 4
// Beschicken der Stanzmaschine mit Transportband 51 663
// durch einen Industry Robot (Knickarm- oder Sulenrobot) im Simultanbetrieb
// frmThreadMain : MainThread Steuerung des Betriebsablaufs, Bedieneroberflche
// RobThread     : Thread     Beschickung durch Industry Robot
// StanzThread   : Thread     Betrieb der Stanzmaschine
// umFish30.DLL  : Thread     Zugriff auf das Interface, durch MultiMediaTimer gesteuert
//
// Modell : 2 x Intelligent Interface an COM1 und COM2
//          alternativ : Intelligent Interface mit Extension Module, in diesem Fall
//                       ist nur eine TFishFace Instanz erforderlich,
//                       die Modellkonstanten mssen entsprechend korrigiert werden.
// Interface Belegung : ftS / COM1 Stanze, ftR / COM2 Industry Robot
// Zum Aufbau der Stanze siehe Stanze.HTM, Schaltung siehe Modellkonstanten
// Der Industry Robot (Sulen/KnickarmRobot) mu nach Handbuch aufgebaut sein.
// Schaltung siehe Modellkonstanten, End-/Impulstaster E1/2, 3/4, 5/6, 7/8
// Robot und Stanze sollten fest miteinander verbunden sein.
//
// Der Bewegungsablauf des Robots wurde nur angedeutet und sollte nach Gegebenheiten
// und eigenen Vorstellungen ausgebaut werden. Dabei sollte bercksichtigt werden,
// dass alle vier Motoren simultan laufen knnen. Bei einem Teilschritt knnen also
// mehrere ftR->SetMotor einem ftR->WaitForMotors vorangehen (siehe auch Home)
//
// --------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "RobStanzeMain.h"

#pragma package(smart_init)
#pragma resource "*.dfm"

// ---------- Modellkonstanten ----------------------------------------------------------

const int
    mBand   = 1, cbVor   = 1, cbRuck  = 2,
    ePhotoV = 1, ePhotoS = 2, mLampen = 3,
    mStanze = 2, eOben   = 3, eUnten  = 4, csAb     = 1, csAuf = 2,
    mSaule  = 1, mArmV   = 3, mArmH   = 2, mGreifer = 4;

// ----------- Globale Variable ---------------------------------------------------------

TfrmThreadMain *frmThreadMain;                // --- Globaler Pointer auf Form
TFishFace      *ftR;                          // --- Globaler Pointer Robot
TFishFace      *ftS;                          // --- Globaler Pointer Stanze

// ---------- Globale Events ------------------------------------------------------------

TEvent *RobReady    = new TEvent(0, 0, 0, "");   // --- Globales Event
                                                 //     standard Security, autoReset,
                                                 //     not signaled, no Name
TEvent *StanzeReady = new TEvent(0, 0, 0, "");


// --------------------- TRobThread -----------------------------------------------------

__fastcall TRobThread::TRobThread(bool CreateSuspended)
        : TThread(CreateSuspended)
{
    this->Priority = tpNormal;
}

void __fastcall TRobThread::Execute()
{
    // --- Home -------------------------------------------------------------
    ftR->SetMotor(mSaule,   ftiLinks, 15, 999);
    ftR->SetMotor(mArmV,    ftiLinks, 15, 999);
    ftR->SetMotor(mArmH,    ftiLinks, 15, 999);
    ftR->SetMotor(mGreifer, ftiLinks, 15, 999);
    ftR->WaitForMotors(0, mSaule, mArmV, mArmH, mGreifer);
    ftR->SetMotor(mSaule,   ftiRechts, 15, 10);
    ftR->WaitForMotors(0, mSaule);

    do
    {
        // --- Greifen ----------------------------------------------------------
        frmThreadMain->lblStatusR->Caption = "Greifen";
        ftR->SetMotor(mGreifer, ftiRechts, 15, 10);
        ftR->WaitForMotors(0, mGreifer);

        // --- Zur Ablage -------------------------------------------------------
        frmThreadMain->lblStatusR->Caption = "Zur Ablage";
        ftR->SetMotor(mSaule, ftiRechts, 15, 100);
        ftR->WaitForMotors(0, mSaule);

        // --- Ablegen/Hoch -----------------------------------------------------
        while(!StanzeReady->WaitFor(100) == wrSignaled);
        frmThreadMain->lblStatusR->Caption = "Ablegen";
        ftR->SetMotor(mGreifer, ftiLinks, 15, 999);
        ftR->WaitForMotors(0, mGreifer);
        RobReady->SetEvent();

        // --- Zum Magazin ------------------------------------------------------
        frmThreadMain->lblStatusR->Caption = "Zum Magazin";
        ftR->SetMotor(mSaule, ftiLinks, 15, 100);
        ftR->WaitForMotors(0, mSaule);
    } while(!Terminated);
    ftR->SetMotor(mSaule, ftiRechts, 15, 100);
    ftR->WaitForMotors(0, mSaule);
}


// --------------------- TStanzThread -----------------------------------------------------

__fastcall TStanzThread::TStanzThread(bool CreateSuspended)
        : TThread(CreateSuspended)
{
    this->Priority = tpNormal;
}

void __fastcall TStanzThread::Execute()
{
    ftS->ClearMotors();
    ftS->SetMotor(mStanze, csAuf);
    ftS->WaitForInput(eOben);
    ftS->SetMotor(mStanze, ftiAus);
    ftS->SetMotor(mLampen, ftiEin);
    do
    {
        // --- Wartet auf Rob -------------------------------------------------------
        frmThreadMain->lblStatusS->Caption = "Wartet";
        StanzeReady->SetEvent();
        while(!RobReady->WaitFor(100) == wrSignaled);
        ftS->WaitForLow(ePhotoV);               // --- Test, wenn der Rob nicht liefert
        ftS->Pause(1234);                       //     mit Hand nachreichen.

        // --- Zur Stanze -----------------------------------------------------------
        frmThreadMain->lblStatusS->Caption = "Zur Stanze";
        ftS->SetMotor(mBand, cbVor);
        ftS->WaitForLow(ePhotoS);
        ftS->SetMotor(mBand, ftiAus);

        // --- Stanzen --------------------------------------------------------------
        frmThreadMain->lblStatusS->Caption = "Stanzen";
        ftS->SetMotor(mStanze, csAb);
        ftS->WaitForInput(eUnten);
        ftS->SetMotor(mStanze, ftiAus);
        ftS->Pause(1000);
        ftS->SetMotor(mStanze, csAuf);
        ftS->WaitForInput(eOben);
        ftS->SetMotor(mStanze, ftiAus);

        // --- Zurck ---------------------------------------------------------------
        frmThreadMain->lblStatusS->Caption = "Zurck",
        ftS->SetMotor(mBand, cbRuck);
        ftS->WaitForInput(ePhotoV, false);
        ftS->Pause(300);
        ftS->SetMotor(mBand, ftiAus);
    } while (!Terminated);
    ftS->SetMotor(mLampen, ftiAus);
}


// --- ThreadMain : Konstruktor ---------------------------------------------------------

__fastcall TfrmThreadMain::TfrmThreadMain(TComponent* Owner)
    : TForm(Owner) {}

// --- cmdActionClick : Starten des Stanzbetriebes -------------------------------------

void __fastcall TfrmThreadMain::cmdActionClick(TObject *Sender)
{
     ftR = new TFishFace(             // --- Instanz von FishFace anlegen
                         false,       //     kein AnalogScan
                         false,       //     ohne Slave
                         0);          //     default PollInterval
     ftS = new TFishFace(false, false, 0);

     try
     {
         ftR->OpenInterface("COM2", false);   // Verbindung zum Interface : COM2
         ftS->OpenInterface("COM1", false);   //            Stanzmaschine : COM1
     }
     catch(TFishFaceException eft)
     {
        lblStatusR->Caption = eft.Text;   // --- wenns nicht klappt : Meldung
        return;                          //     Ende START
     }

     cmdAction->Enabled     = false;     // --- Buttons sortieren
     cmdEnde->Caption       = "&HALT";


     RobThread   = new TRobThread(true);       // --- Threads anlegen
     StanzThread = new TStanzThread(true);

     RobThread->OnTerminate   = RobEnde;  // Abschluroutine
     StanzThread->OnTerminate = StanzEnde;

     RobThread->Suspended   = false;       // --- Starten der Threads
     StanzThread->Suspended = false;
}


// --- cmdEndeClick : Beenden des Stanzbetriebes, Programmende bei ENDE ----------------

void __fastcall TfrmThreadMain::cmdEndeClick(TObject *Sender)
{
    if(cmdEnde->Caption == "&HALT")
    {   // --- Beenden des Stanzbetriebes ---------------------------
        lblStatusR->Caption = "Beenden wird eingeleitet";
        RobThread->Terminate();
        StanzeReady->SetEvent();
        RobThread->WaitFor();            //     Warten auf Threadende
        StanzThread->Terminate();
        RobReady->SetEvent();
        StanzThread->WaitFor();          //     Warten auf Threadende

        lblStatusR->Caption = "--- Finito ---";
        Application->ProcessMessages();
        Sleep(555);

        ftR->CloseInterface();           // --- Verbindung zum Interface schlieen
        ftS->CloseInterface();

        cmdAction->Enabled = true;       // --- Buttons gerade rcken
        cmdEnde->Caption = "&ENDE";
    }
    else
    {   // --- Beenden des Programms ------------------------------
        Close();
    }
}

// --- RobEnde : EndRoutine des Threads  -------------------------------

void __fastcall TfrmThreadMain::RobEnde(TObject * Sender)
{
    lblStatusR->Caption = "RobThread : Ende";
    Sleep(500);
}

// --- StanzEnde : EndRoutine des Threads --------------------------------

void __fastcall TfrmThreadMain::StanzEnde(TObject * Sender)
{
    lblStatusS->Caption = "StanzThread : Ende";
    Sleep(500);
}

// --- FormCloseQuery : Programmende sperren, wenn noch Stanzbetrieb ----------------

void __fastcall TfrmThreadMain::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
    if(cmdEnde->Caption == "&HALT") CanClose = false;
}

